@@ -1,31 +1,42 @@
-#
-# This file is part of Games::RailRoad.
-# Copyright (c) 2008 Jerome Quelin, all rights reserved.
-#
-# This program is free software; you can redistribute it and/or modify
-# it under the same terms as Perl itself.
-#
-
-package Games::RailRoad::Node;
-
+# 
+# This file is part of Games-RailRoad
+# 
+# This software is copyright (c) 2008 by Jerome Quelin.
+# 
+# This is free software; you can redistribute it and/or modify it under
+# the same terms as the Perl 5 programming language system itself.
+# 
+use 5.010;
 use strict;
 use warnings;
-use 5.010;
 
+package Games::RailRoad::Node;
+BEGIN {
+  $Games::RailRoad::Node::VERSION = '1.101330';
+}
+# ABSTRACT: a node object
+
+use Moose;
+use MooseX::Has::Sugar;
+use MooseX::SemiAffordanceAccessor;
 use UNIVERSAL::require;
 
-use base qw{ Class::Accessor::Fast };
-__PACKAGE__->mk_accessors( qw{ position } );
+
+# -- attributes
 
 
-# -- PUBLIC METHODS
+has position => ( ro, isa=>'Games::RailRoad::Vector', required );
+
+
+# -- constructor & initializers
+
+
+# provided by moose
+
+
+# -- public methods
+
 
-#
-# $node->connect( $dir );
-#
-# connect the node to the node in the wanted $dir. return undef if it
-# isn't possible.
-#
 sub connect {
     my ($self, $dir) = @_;
 
@@ -47,12 +58,7 @@ sub connect {
 }
 
 
-#
-# $node->connectable( $dir );
-#
-# return true if the node can be connected to the node in the wanted
-# $dir. return false otherwise.
-#
+
 sub connectable {
     my ($self, $dir) = @_;
     my $map = $self->_transform_map;
@@ -60,11 +66,7 @@ sub connectable {
 }
 
 
-#
-# my @dirs = $node->connections;
-#
-# return a list of dirs in which the node is connected.
-#
+
 sub connections {
     my ($self) = @_;
     my $pkg = ref $self;
@@ -74,23 +76,15 @@ sub connections {
 }
 
 
-#
-# $node->delete( $canvas );
-#
-# request $node to delete itself from $canvas.
-#
+
 sub delete {
     my ($self, $canvas) = @_;
     my $pos = $self->position;
     $canvas->delete("$pos");
 }
 
-#
-# $node->draw( $canvas, $tilelen );
-#
-# request $node to draw itself on $canvas, assuming that each
-# square has a length of $tilelen.
-#
+
+
 sub draw {
     my ($self, $canvas, $tilelen) = @_;
     $self->delete($canvas);
@@ -103,31 +97,20 @@ sub draw {
 }
 
 
-#
-# my $to = $node->next_dir( $from );
-#
-# when $node is reached by a train, this method will return the next
-# direction to head to, assuming the train was coming from $from.
-#
-# can return undef if there's no such $from configured, or if the node
-# is a dead-end.
-#
+
 sub next_dir {
     my ($self, $from) = @_;
+    # each node class is defining a _next_map() method that returns a
+    # hashref of {from=>to}
     return $self->_next_map->{$from};
 }
 
 
-#
-# $node->switch;
-#
-# request a node to change its exit, if possible. this is a no-op for
-# most nodes, except games::railroad::node::switch::*
-#
+
 sub switch {}
 
 
-# -- PRIVATE METHODS
+# -- private methods
 
 #
 # $node->_draw_segment( $segment, $canvas, $tilelen )
@@ -140,8 +123,8 @@ sub _draw_segment {
     my ($self, $segment, $canvas, $tilelen) = @_;
 
     my $pos  = $self->position;
-    my $col1 = $pos->x;
-    my $row1 = $pos->y;
+    my $col1 = $pos->posx;
+    my $row1 = $pos->posy;
     my ($col2, $row2) = ($col1, $row1);
 
     # since each node is overlapping with the surrounding ones, we just
@@ -153,8 +136,8 @@ sub _draw_segment {
     # create the line.
     my $tags = [ "$pos", "$pos-$end" ];
     $canvas->createLine(
-        $tilelen * $pos->x, $tilelen * $pos->y,
-        $tilelen * $end->x, $tilelen * $end->y,
+        $tilelen * $pos->posx, $tilelen * $pos->posy,
+        $tilelen * $end->posx, $tilelen * $end->posy,
         -tags=>$tags
     );
 
@@ -162,8 +145,8 @@ sub _draw_segment {
     my $div    = 3;
     my $radius = 1;
     foreach my $i ( 0 .. $div ) {
-        my $x = $tilelen * ( $pos->x + $move->x * $i / $div );
-        my $y = $tilelen * ( $pos->y + $move->y * $i / $div );
+        my $x = $tilelen * ( $pos->posx + $move->posx * $i / $div );
+        my $y = $tilelen * ( $pos->posy + $move->posy * $i / $div );
         $canvas->createOval(
             $x-$radius, $y-$radius,
             $x+$radius, $y+$radius,
@@ -199,81 +182,69 @@ sub _transform_map {
 
 
 1;
-__END__
 
 
+=pod
+
 =head1 NAME
 
 Games::RailRoad::Node - a node object
 
+=head1 VERSION
 
+version 1.101330
 
 =head1 DESCRIPTION
 
-C<Games::RailRoad::Node> provides a node object. This is the base class
-for the following classes:
+This module provides a node object. This is the base class for the
+following classes:
 
 =over 4
 
 =item *
 
-C<Games::RailRoad::Node::Half> is a node with only one segment, from the
+L<Games::RailRoad::Node::Half> is a node with only one segment, from the
 center to one of the 8 extremities of a square.
 
 =item *
 
-C<Games::RailRoad::Node::Straight> is a node with two segments, linking
+L<Games::RailRoad::Node::Straight> is a node with two segments, linking
 two of the 8 extremities of a square.
 
 =item *
 
-C<Games::RailRoad::Node::Switch> is a node with three segments, linking
+L<Games::RailRoad::Node::Switch> is a node with three segments, linking
 three of the 8 extremities of a square through the center. The I<active>
 segment taken by a train riding this node can switch between two of the
 segments.
 
 =item *
 
-C<Games::RailRoad::Node::Cross> is a node with four segments: two
+L<Games::RailRoad::Node::Cross> is a node with four segments: two
 straight lines crossing in the center of the square.
 
 =back
 
-
 Each of those classes also has subclasses, one for each configuration
 allowed. They are named after each of the existing extremity of the
 square linked (in uppercase), sorted and separated by underscore (C<_>).
-For example: C<Games::RailRoad::Node::Switch::N_S_SE>.
-
+For example: L<Games::RailRoad::Node::Switch::N_S_SE>.
 
 Note that each segment coming out of a node belongs to 2 different
 (adjacent) nodes.
 
+=head1 ATTRIBUTES
 
+=head2 position
 
-=head1 CONSTRUCTOR
+The node position (a L<Games::RailRoad::Vector>).
+
+=head1 METHODS
 
 =head2 my $node = Games::RailRoad::Node->new( \%opts );
 
 Create a new node object. One can pass a hash reference with the
-following keys:
-
-=over 4
-
-=item col => $col
-
-the column of the canvas where the node is.
-
-=item row => $row
-
-the row of the canvas where the node is.
-
-
-=back
-
-
-
-=head1 PUBLIC METHODS
+available attributes.
 
 =head2 $node->connect( $dir );
 
@@ -284,39 +255,29 @@ C<$dir> should be one of C<nw>, C<n>, C<ne>, C<w>, C<e>, C<sw>, C<s>,
 C<se>. Of course, other values are accepted but won't result in a node
 extension.
 
-
-
 =head2 $node->connectable( $dir );
 
 Return true if C<$node> can be connected to the wanted C<$dir>. Return
 false otherwise.
 
 C<$dir> should be one of C<nw>, C<n>, C<ne>, C<w>, C<e>, C<sw>, C<s>,
-C<se>. Of course, other values are accepted but will return always
+C<se>. Of course, other values are accepted but will always return
 false.
 
-
-
 =head2 my @dirs = $node->connections;
 
 Return a list of dirs in which the node is connected.
 
-
-
 =head2 $node->delete( $canvas );
 
 Request C<$node> to remove itself from C<$canvas>.
 
-
-
 =head2 $node->draw( $canvas, $tilelen );
 
 Request C<$node> to draw itself on C<$canvas>, assuming that each square
 has a length of C<$tilelen>. Note that this method calls the C<delete()>
 method first.
 
-
-
 =head2 my $to = $node->next_dir( $from );
 
 When C<$node> is reached by a train, this method will return the next
@@ -325,34 +286,25 @@ direction to head to, assuming the train was coming from C<$from>.
 Note that the method can return undef if there's no such C<$from>
 configured, or if the node is a dead-end.
 
-
-
 =head2 $node->switch;
 
 Request a node to change its exit, if possible. This is a no-op for most
 nodes, except C<Games::Railroad::Node::Switch::*>.
 
-
-
-=head1 SEE ALSO
-
-L<Games::RailRoad>.
-
-
-
 =head1 AUTHOR
 
-Jerome Quelin, C<< <jquelin at cpan.org> >>
+  Jerome Quelin
 
+=head1 COPYRIGHT AND LICENSE
 
+This software is copyright (c) 2008 by Jerome Quelin.
 
-=head1 COPYRIGHT & LICENSE
-
-Copyright (c) 2008 Jerome Quelin, all rights reserved.
-
-This program is free software; you can redistribute it and/or modify
-it under the same terms as Perl itself.
-
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
 
 =cut
 
+
+__END__
+
+
